@@ -32,12 +32,35 @@ pub fn impl_ltv_collection(input: DeriveInput) -> proc_macro2::TokenStream {
     })
     .collect();
 
+    let byte_order = match attrs.byte_order {
+        ByteOrderOption::BE => quote! { {::ltv::ByteOrder::BE} },
+        ByteOrderOption::LE => quote! { {::ltv::ByteOrder::LE} },
+        ByteOrderOption::None => quote! { T },
+    };
+
+    let byte_order_impl = match attrs.byte_order {
+        ByteOrderOption::BE => quote! { impl LTVItem<{::ltv::ByteOrder::BE}> },
+        ByteOrderOption::LE => quote! {impl LTVItem<{::ltv::ByteOrder::LE}> },
+        ByteOrderOption::None => quote! {impl<const ED: ::ltv::ByteOrder> LTVItem<ED> },
+    };
+    let len_size = attrs.length_size.unwrap_or(1) as usize;
+
     let from_ltv_fn = {
         let object_match_branches = variants.iter().map(|info| {
             let inner_ltv = &info.inner_data;
             let branch_name = &info.enum_field;
+            let branch_err_name = format!("{}::{}",enum_ident, branch_name);
             quote! {
-                #inner_ltv::OBJECT_ID => Ok(Self::#branch_name(#inner_ltv::from_ltv(#inner_ltv::OBJECT_ID, data)?))
+                <#inner_ltv as ::ltv::LTVObject<#len_size>>::OBJECT_ID =>
+                    Ok(Self::#branch_name(
+                        <#inner_ltv as ::ltv::LTVItem<#byte_order>>::from_ltv(
+                            <#inner_ltv as ::ltv::LTVObject<#len_size>>::OBJECT_ID, data
+                        ).map_err(|e| ::ltv::LTVError::InnerParseError(
+                            e.into(),
+                            format!(#branch_err_name)
+                        ))?
+                    )
+                )
             }
         });
 
@@ -56,8 +79,9 @@ pub fn impl_ltv_collection(input: DeriveInput) -> proc_macro2::TokenStream {
     let to_ltv_fn = {
         let object_match_branches = variants.iter().map(|info| {
             let branch_name = &info.enum_field;
+            let inner_ltv = &info.inner_data;
             quote! {
-                Self::#branch_name(v) => v.to_ltv()
+                Self::#branch_name(v) => <#inner_ltv as LTVItem<#byte_order>>::to_ltv(v)
             }
         });
 
@@ -73,8 +97,9 @@ pub fn impl_ltv_collection(input: DeriveInput) -> proc_macro2::TokenStream {
     let to_ltv_object_branches = {
         let object_match_branches = variants.iter().map(|info| {
             let branch_name = &info.enum_field;
+            let inner_ltv = &info.inner_data;
             quote! {
-                Self::#branch_name(v) => v.to_ltv_object()
+                Self::#branch_name(v) => <#inner_ltv as LTVObjectConvertable<#byte_order, #len_size>>::to_ltv_object(v)
             }
         });
 
@@ -85,20 +110,6 @@ pub fn impl_ltv_collection(input: DeriveInput) -> proc_macro2::TokenStream {
         }
     };
 
-    let byte_order = match attrs.byte_order {
-        ByteOrderOption::BE => quote! { {::ltv::ByteOrder::BE} },
-        ByteOrderOption::LE => quote! { {::ltv::ByteOrder::LE} },
-        ByteOrderOption::None => quote! { T },
-    };
-
-    let byte_order_impl = match attrs.byte_order {
-        ByteOrderOption::BE => quote! { impl LTVItem<{::ltv::ByteOrder::BE}> },
-        ByteOrderOption::LE => quote! {impl LTVItem<{::ltv::ByteOrder::LE}> },
-        ByteOrderOption::None => quote! {impl<const ED: ::ltv::ByteOrder> LTVItem<ED> },
-    };
-
-    let len_size = attrs.length_size.unwrap_or(1) as usize;
-
     let e = quote! {
         #[automatically_derived]
         #byte_order_impl for #enum_ident {
@@ -108,9 +119,8 @@ pub fn impl_ltv_collection(input: DeriveInput) -> proc_macro2::TokenStream {
 
         impl <'a> LTVObjectConvertable<'a, #byte_order, #len_size> for #enum_ident {
             fn from_ltv_object(data: &'a [u8]) -> LTVResult<Self> {
-                use ::ltv::LTVReader;
-                let (_, obj_id, data) = LTVReader::<'a, #byte_order, #len_size>::parse_ltv(data)?;
-                Ok(Self::from_ltv(obj_id, data)?)
+                let (_, obj_id, data) = ::ltv::LTVReader::<'a, #byte_order, #len_size>::parse_ltv(data)?;
+                Ok(<Self as LTVItem<#byte_order>>::from_ltv(obj_id, data)?)
             }
 
             fn to_ltv_object(&self) -> Vec<u8> {
@@ -118,8 +128,13 @@ pub fn impl_ltv_collection(input: DeriveInput) -> proc_macro2::TokenStream {
             }
         }
     };
-
-    //use std::fs;
-    //fs::write(format!("object_impl_{}.rs", &enum_ident), e.to_string()).ok();
+    /*
+    use std::fs;
+    fs::write(
+        format!("target/object_impl_{}.rs", &enum_ident),
+        e.to_string(),
+    )
+    .ok();
+    */
     e
 }
